@@ -26,6 +26,8 @@ import {
   VariableOption,
   defaultDataQueryKind,
   AdHocFilterWithLabels,
+  SwitchVariableKind,
+  defaultIntervalVariableSpec,
 } from '@grafana/schema/dist/esm/schema/dashboard/v2';
 import { getDefaultDatasource } from 'app/features/dashboard/api/ResponseTransformers';
 
@@ -33,7 +35,7 @@ import { getIntervalsQueryFromNewIntervalModel } from '../utils/utils';
 
 import { DSReferencesMapping } from './DashboardSceneSerializer';
 import { getDataSourceForQuery } from './layoutSerializers/utils';
-import { getDataQueryKind, getDataQuerySpec, getElementDatasource } from './transformSceneToSaveModelSchemaV2';
+import { getDataQueryKind, getElementDatasource } from './transformSceneToSaveModelSchemaV2';
 import {
   transformVariableRefreshToEnum,
   transformVariableHideToEnum,
@@ -64,12 +66,11 @@ export function sceneVariablesSetToVariables(set: SceneVariables, keepQueryOptio
 
     if (sceneUtils.isQueryVariable(variable)) {
       let options: VariableOption[] = [];
-      // Not sure if we actually have to still support this option given
-      // that it's not exposed in the UI
-      if (transformVariableRefreshToEnum(variable.state.refresh) === 'never' || keepQueryOptions) {
+      if (keepQueryOptions) {
         options = variableValueOptionsToVariableOptions(variable.state);
       }
-      variables.push({
+      const datasource = getElementDatasource(set, variable, 'variable');
+      const variableObj: VariableModel = {
         ...commonProperties,
         current: {
           // @ts-expect-error
@@ -80,23 +81,32 @@ export function sceneVariablesSetToVariables(set: SceneVariables, keepQueryOptio
         options,
         query: variable.state.query,
         definition: variable.state.definition,
-        datasource: getElementDatasource(set, variable, 'variable'),
         sort: variable.state.sort,
         refresh: variable.state.refresh,
         regex: variable.state.regex,
+        regexApplyTo: variable.state.regexApplyTo,
         allValue: variable.state.allValue,
         includeAll: variable.state.includeAll,
         multi: variable.state.isMulti,
-        allowCustomValue: variable.state.allowCustomValue,
+        ...(variable.state.allowCustomValue !== undefined && { allowCustomValue: variable.state.allowCustomValue }),
         skipUrlSync: variable.state.skipUrlSync,
         staticOptions: variable.state.staticOptions?.map((option) => ({
           text: option.label,
           value: String(option.value),
         })),
         staticOptionsOrder: variable.state.staticOptionsOrder,
-      });
+      };
+      // Only add datasource if it exists and is not empty
+      if (datasource && Object.keys(datasource).length > 0 && (datasource.uid || datasource.type)) {
+        variableObj.datasource = datasource;
+      }
+      variables.push(variableObj);
     } else if (sceneUtils.isCustomVariable(variable)) {
-      variables.push({
+      let options: VariableOption[] = [];
+      if (keepQueryOptions) {
+        options = variableValueOptionsToVariableOptions(variable.state);
+      }
+      const customVariable: VariableModel = {
         ...commonProperties,
         current: {
           // @ts-expect-error
@@ -104,13 +114,14 @@ export function sceneVariablesSetToVariables(set: SceneVariables, keepQueryOptio
           // @ts-expect-error
           value: variable.state.value,
         },
-        options: variableValueOptionsToVariableOptions(variable.state),
+        options,
         query: variable.state.query,
         multi: variable.state.isMulti,
         allValue: variable.state.allValue,
         includeAll: variable.state.includeAll,
-        allowCustomValue: variable.state.allowCustomValue,
-      });
+        ...(variable.state.allowCustomValue !== undefined && { allowCustomValue: variable.state.allowCustomValue }),
+      };
+      variables.push(customVariable);
     } else if (sceneUtils.isDataSourceVariable(variable)) {
       variables.push({
         ...commonProperties,
@@ -127,11 +138,12 @@ export function sceneVariablesSetToVariables(set: SceneVariables, keepQueryOptio
         multi: variable.state.isMulti,
         allValue: variable.state.allValue,
         includeAll: variable.state.includeAll,
-        allowCustomValue: variable.state.allowCustomValue,
+        ...(variable.state.allowCustomValue !== undefined && { allowCustomValue: variable.state.allowCustomValue }),
       });
     } else if (sceneUtils.isConstantVariable(variable)) {
       variables.push({
         ...commonProperties,
+        type: 'constant', // Explicitly set type to constant
         current: {
           // @ts-expect-error
           value: variable.state.value,
@@ -151,7 +163,9 @@ export function sceneVariablesSetToVariables(set: SceneVariables, keepQueryOptio
           value: variable.state.value,
         },
         query: intervals,
-        refresh: variable.state.refresh,
+        // V2 schema mandates refresh: "onTimeRangeChanged" for interval variables,
+        // which maps to OldVariableRefresh.onTimeRangeChanged (2) in V1
+        refresh: variable.state.refresh ?? OldVariableRefresh.onTimeRangeChanged,
         options: variable.state.intervals.map((interval) => ({
           value: interval,
           text: interval,
@@ -198,17 +212,36 @@ export function sceneVariablesSetToVariables(set: SceneVariables, keepQueryOptio
           value: variable.state.value,
         },
         defaultValue: defaultVariableOption,
-        allowCustomValue: variable.state.allowCustomValue,
+        ...(variable.state.allowCustomValue !== undefined && { allowCustomValue: variable.state.allowCustomValue }),
       });
     } else if (sceneUtils.isAdHocVariable(variable)) {
-      variables.push({
+      const adhocVariable: VariableModel = {
         ...commonProperties,
         datasource: variable.state.datasource,
-        allowCustomValue: variable.state.allowCustomValue,
         // @ts-expect-error
         baseFilters: variable.state.baseFilters || [],
         filters: [...validateFiltersOrigin(variable.state.originFilters), ...variable.state.filters],
         defaultKeys: variable.state.defaultKeys,
+        ...(variable.state.allowCustomValue !== undefined && { allowCustomValue: variable.state.allowCustomValue }),
+      };
+      variables.push(adhocVariable);
+    } else if (sceneUtils.isSwitchVariable(variable)) {
+      variables.push({
+        ...commonProperties,
+        current: {
+          value: variable.state.value,
+          text: variable.state.value,
+        },
+        options: [
+          {
+            value: variable.state.enabledValue,
+            text: variable.state.enabledValue,
+          },
+          {
+            value: variable.state.disabledValue,
+            text: variable.state.disabledValue,
+          },
+        ],
       });
     } else if (variable.state.type === 'system') {
       // Not persisted
@@ -264,6 +297,7 @@ export function sceneVariablesSetToSchemaV2Variables(
   | ConstantVariableKind
   | GroupByVariableKind
   | AdhocVariableKind
+  | SwitchVariableKind
 > {
   let variables: Array<
     | QueryVariableKind
@@ -274,6 +308,7 @@ export function sceneVariablesSetToSchemaV2Variables(
     | ConstantVariableKind
     | GroupByVariableKind
     | AdhocVariableKind
+    | SwitchVariableKind
   > = [];
 
   for (const variable of set.state.variables) {
@@ -294,10 +329,10 @@ export function sceneVariablesSetToSchemaV2Variables(
     };
 
     let options: VariableOption[] = [];
+
+    // Query variable
     if (sceneUtils.isQueryVariable(variable)) {
-      // Not sure if we actually have to still support this option given
-      // that it's not exposed in the UI
-      if (transformVariableRefreshToEnum(variable.state.refresh) === 'never' || keepQueryOptions) {
+      if (keepQueryOptions) {
         options = variableValueOptionsToVariableOptions(variable.state);
       }
       const query = variable.state.query;
@@ -308,27 +343,30 @@ export function sceneVariablesSetToSchemaV2Variables(
         dataQuery = {
           kind: 'DataQuery',
           version: defaultDataQueryKind().version,
-          group: datasource?.type ?? getDataQueryKind(query),
+          group: datasource?.type || getDataQueryKind(query),
           ...(datasource?.uid && {
             datasource: {
               name: datasource.uid,
             },
           }),
-          spec: getDataQuerySpec(query),
+          spec: query,
         };
       } else {
+        // Only include LEGACY_STRING_VALUE_KEY if query is a non-empty string
+        const spec: Record<string, string> = {};
+        if (query) {
+          spec[LEGACY_STRING_VALUE_KEY] = query;
+        }
         dataQuery = {
           kind: 'DataQuery',
           version: defaultDataQueryKind().version,
-          group: datasource?.type ?? getDataQueryKind(query),
+          group: datasource?.type || getDataQueryKind(query),
           ...(datasource?.uid && {
             datasource: {
               name: datasource.uid,
             },
           }),
-          spec: {
-            [LEGACY_STRING_VALUE_KEY]: query,
-          },
+          spec,
         };
       }
       const queryVariable: QueryVariableKind = {
@@ -341,7 +379,8 @@ export function sceneVariablesSetToSchemaV2Variables(
           definition: variable.state.definition,
           sort: transformSortVariableToEnum(variable.state.sort),
           refresh: transformVariableRefreshToEnum(variable.state.refresh),
-          regex: variable.state.regex,
+          regex: variable.state.regex ?? '',
+          regexApplyTo: variable.state.regexApplyTo ?? 'value',
           allValue: variable.state.allValue,
           includeAll: variable.state.includeAll || false,
           multi: variable.state.isMulti || false,
@@ -355,14 +394,15 @@ export function sceneVariablesSetToSchemaV2Variables(
         },
       };
       variables.push(queryVariable);
+
+      // Custom variable
     } else if (sceneUtils.isCustomVariable(variable)) {
-      options = variableValueOptionsToVariableOptions(variable.state);
       const customVariable: CustomVariableKind = {
         kind: 'CustomVariable',
         spec: {
           ...commonProperties,
           current: currentVariableOption,
-          options,
+          options: [],
           query: variable.state.query,
           multi: variable.state.isMulti || false,
           allValue: variable.state.allValue,
@@ -371,6 +411,8 @@ export function sceneVariablesSetToSchemaV2Variables(
         },
       };
       variables.push(customVariable);
+
+      // Datasource variable
     } else if (sceneUtils.isDataSourceVariable(variable)) {
       const datasourceVariable: DatasourceVariableKind = {
         kind: 'DatasourceVariable',
@@ -378,9 +420,9 @@ export function sceneVariablesSetToSchemaV2Variables(
           ...commonProperties,
           current: currentVariableOption,
           options: [],
-          regex: variable.state.regex,
+          regex: variable.state.regex ?? '',
           refresh: 'onDashboardLoad',
-          pluginId: variable.state.pluginId,
+          pluginId: variable.state.pluginId ?? getDefaultDatasource().type,
           multi: variable.state.isMulti || false,
           includeAll: variable.state.includeAll || false,
           allowCustomValue: variable.state.allowCustomValue ?? true,
@@ -392,21 +434,24 @@ export function sceneVariablesSetToSchemaV2Variables(
       }
 
       variables.push(datasourceVariable);
+
+      // Constant variable
     } else if (sceneUtils.isConstantVariable(variable)) {
+      const value = variable.state.value ? String(variable.state.value) : '';
       const constantVariable: ConstantVariableKind = {
         kind: 'ConstantVariable',
         spec: {
           ...commonProperties,
           current: {
-            ...currentVariableOption,
-            // Constant variable doesn't use text state
-            text: String(variable.state.value),
+            text: value,
+            value: value,
           },
-          // @ts-expect-error
-          query: variable.state.value,
+          query: value,
         },
       };
       variables.push(constantVariable);
+
+      // Interval variable
     } else if (sceneUtils.isIntervalVariable(variable)) {
       const intervals = getIntervalsQueryFromNewIntervalModel(variable.state.intervals);
       const intervalVariable: IntervalVariableKind = {
@@ -419,22 +464,24 @@ export function sceneVariablesSetToSchemaV2Variables(
             text: variable.state.value,
           },
           query: intervals,
-          refresh: 'onTimeRangeChanged',
+          refresh: defaultIntervalVariableSpec().refresh,
           options: variable.state.intervals.map((interval) => ({
             value: interval,
             text: interval,
             selected: interval === variable.state.value,
           })),
-          auto: variable.state.autoEnabled,
-          auto_min: variable.state.autoMinInterval,
-          auto_count: variable.state.autoStepCount,
+          auto: variable.state.autoEnabled ?? defaultIntervalVariableSpec().auto,
+          auto_min: variable.state.autoMinInterval ?? defaultIntervalVariableSpec().auto_min,
+          auto_count: variable.state.autoStepCount ?? defaultIntervalVariableSpec().auto_count,
         },
       };
       variables.push(intervalVariable);
+
+      // Textbox variable
     } else if (sceneUtils.isTextBoxVariable(variable)) {
       const current = {
-        text: variable.state.value,
-        value: variable.state.value,
+        text: variable.state.value ?? '',
+        value: variable.state.value ?? '',
       };
 
       const textBoxVariable: TextVariableKind = {
@@ -442,11 +489,13 @@ export function sceneVariablesSetToSchemaV2Variables(
         spec: {
           ...commonProperties,
           current,
-          query: variable.state.value,
+          query: variable.state.value ?? '',
         },
       };
 
       variables.push(textBoxVariable);
+
+      // Groupby variable
     } else if (sceneUtils.isGroupByVariable(variable) && config.featureToggles.groupByVariable) {
       options = variableValueOptionsToVariableOptions(variable.state);
 
@@ -483,6 +532,8 @@ export function sceneVariablesSetToSchemaV2Variables(
         },
       };
       variables.push(groupVariable);
+
+      // Adhoc variable
     } else if (sceneUtils.isAdHocVariable(variable)) {
       const ds = getDataSourceForQuery(
         variable.state.datasource,
@@ -503,11 +554,24 @@ export function sceneVariablesSetToSchemaV2Variables(
             ...validateFiltersOrigin(variable.state.originFilters),
             ...validateFiltersOrigin(variable.state.filters),
           ],
-          defaultKeys: variable.state.defaultKeys || [], //FIXME what is the default value?
+          defaultKeys: variable.state.defaultKeys || [],
           allowCustomValue: variable.state.allowCustomValue ?? true,
         },
       };
       variables.push(adhocVariable);
+
+      // Switch variable
+    } else if (sceneUtils.isSwitchVariable(variable)) {
+      const switchVariable: SwitchVariableKind = {
+        kind: 'SwitchVariable',
+        spec: {
+          ...commonProperties,
+          current: variable.state.value,
+          enabledValue: variable.state.enabledValue,
+          disabledValue: variable.state.disabledValue,
+        },
+      };
+      variables.push(switchVariable);
     } else if (variable.state.type === 'system') {
       // Do nothing
     } else {
